@@ -16,27 +16,34 @@
 
 """Compute v2 Aggregate action implementations"""
 
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
+from openstackclient.i18n import _
+
+
+LOG = logging.getLogger(__name__)
 
 
 class AddAggregateHost(command.ShowOne):
-    """Add host to aggregate"""
+    _description = _("Add host to aggregate")
 
     def get_parser(self, prog_name):
         parser = super(AddAggregateHost, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Aggregate (name or ID)',
+            help=_("Aggregate (name or ID)")
         )
         parser.add_argument(
             'host',
             metavar='<host>',
-            help='Host to add to <aggregate>',
+            help=_("Host to add to <aggregate>")
         )
         return parser
 
@@ -55,26 +62,26 @@ class AddAggregateHost(command.ShowOne):
 
 
 class CreateAggregate(command.ShowOne):
-    """Create a new aggregate"""
+    _description = _("Create a new aggregate")
 
     def get_parser(self, prog_name):
         parser = super(CreateAggregate, self).get_parser(prog_name)
         parser.add_argument(
             "name",
             metavar="<name>",
-            help="New aggregate name",
+            help=_("New aggregate name")
         )
         parser.add_argument(
             "--zone",
             metavar="<availability-zone>",
-            help="Availability zone name",
+            help=_("Availability zone name")
         )
         parser.add_argument(
             "--property",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help='Property to add to this aggregate '
-                 '(repeat option to set multiple properties)',
+            help=_("Property to add to this aggregate "
+                   "(repeat option to set multiple properties)")
         )
         return parser
 
@@ -98,29 +105,41 @@ class CreateAggregate(command.ShowOne):
 
 
 class DeleteAggregate(command.Command):
-    """Delete an existing aggregate"""
+    _description = _("Delete existing aggregate(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteAggregate, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Aggregate to delete (name or ID)',
+            nargs='+',
+            help=_("Aggregate(s) to delete (name or ID)")
         )
         return parser
 
     def take_action(self, parsed_args):
-
         compute_client = self.app.client_manager.compute
-        data = utils.find_resource(
-            compute_client.aggregates,
-            parsed_args.aggregate,
-        )
-        compute_client.aggregates.delete(data.id)
+        result = 0
+        for a in parsed_args.aggregate:
+            try:
+                data = utils.find_resource(
+                    compute_client.aggregates, a)
+                compute_client.aggregates.delete(data.id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete aggregate with name or "
+                          "ID '%(aggregate)s': %(e)s")
+                          % {'aggregate': a, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.aggregate)
+            msg = (_("%(result)s of %(total)s aggregates failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListAggregate(command.Lister):
-    """List all aggregates"""
+    _description = _("List all aggregates")
 
     def get_parser(self, prog_name):
         parser = super(ListAggregate, self).get_parser(prog_name)
@@ -128,7 +147,8 @@ class ListAggregate(command.Lister):
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output')
+            help=_("List additional fields in output")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -168,19 +188,19 @@ class ListAggregate(command.Lister):
 
 
 class RemoveAggregateHost(command.ShowOne):
-    """Remove host from aggregate"""
+    _description = _("Remove host from aggregate")
 
     def get_parser(self, prog_name):
         parser = super(RemoveAggregateHost, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Aggregate (name or ID)',
+            help=_("Aggregate (name or ID)")
         )
         parser.add_argument(
             'host',
             metavar='<host>',
-            help='Host to remove from <aggregate>',
+            help=_("Host to remove from <aggregate>")
         )
         return parser
 
@@ -202,31 +222,39 @@ class RemoveAggregateHost(command.ShowOne):
 
 
 class SetAggregate(command.Command):
-    """Set aggregate properties"""
+    _description = _("Set aggregate properties")
 
     def get_parser(self, prog_name):
         parser = super(SetAggregate, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Aggregate to modify (name or ID)',
+            help=_("Aggregate to modify (name or ID)")
         )
         parser.add_argument(
             '--name',
             metavar='<name>',
-            help='Set aggregate name',
+            help=_("Set aggregate name")
         )
         parser.add_argument(
             "--zone",
             metavar="<availability-zone>",
-            help="Set availability zone name",
+            help=_("Set availability zone name")
         )
         parser.add_argument(
             "--property",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help='Property to set on <aggregate> '
-                 '(repeat option to set multiple properties)',
+            help=_("Property to set on <aggregate> "
+                   "(repeat option to set multiple properties)")
+        )
+        parser.add_argument(
+            "--no-property",
+            dest="no_property",
+            action="store_true",
+            help=_("Remove all properties from <aggregate> "
+                   "(specify both --property and --no-property to "
+                   "overwrite the current properties)"),
         )
         return parser
 
@@ -249,22 +277,35 @@ class SetAggregate(command.Command):
                 kwargs
             )
 
+        set_property = {}
+        if parsed_args.no_property:
+            # NOTE(RuiChen): "availability_zone" is removed from response of
+            #                aggregate show and create commands, don't see it
+            #                anywhere, so pop it, avoid the unexpected server
+            #                exception(can't unset the availability zone from
+            #                aggregate metadata in nova).
+            set_property.update({key: None
+                                 for key in aggregate.metadata.keys()
+                                 if key != 'availability_zone'})
         if parsed_args.property:
+            set_property.update(parsed_args.property)
+
+        if set_property:
             compute_client.aggregates.set_metadata(
                 aggregate,
-                parsed_args.property
+                set_property
             )
 
 
 class ShowAggregate(command.ShowOne):
-    """Display aggregate details"""
+    _description = _("Display aggregate details")
 
     def get_parser(self, prog_name):
         parser = super(ShowAggregate, self).get_parser(prog_name)
         parser.add_argument(
             'aggregate',
             metavar='<aggregate>',
-            help='Aggregate to display (name or ID)',
+            help=_("Aggregate to display (name or ID)")
         )
         return parser
 
@@ -293,22 +334,21 @@ class ShowAggregate(command.ShowOne):
 
 
 class UnsetAggregate(command.Command):
-    """Unset aggregate properties"""
+    _description = _("Unset aggregate properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetAggregate, self).get_parser(prog_name)
         parser.add_argument(
             "aggregate",
             metavar="<aggregate>",
-            help="Aggregate to modify (name or ID)",
+            help=_("Aggregate to modify (name or ID)")
         )
         parser.add_argument(
             "--property",
             metavar="<key>",
             action='append',
-            help='Property to remove from aggregate '
-                 '(repeat option to remove multiple properties)',
-            required=True,
+            help=_("Property to remove from aggregate "
+                   "(repeat option to remove multiple properties)")
         )
         return parser
 
@@ -318,6 +358,9 @@ class UnsetAggregate(command.Command):
             compute_client.aggregates,
             parsed_args.aggregate)
 
-        unset_property = {key: None for key in parsed_args.property}
-        compute_client.aggregates.set_metadata(aggregate,
-                                               unset_property)
+        unset_property = {}
+        if parsed_args.property:
+            unset_property.update({key: None for key in parsed_args.property})
+        if unset_property:
+            compute_client.aggregates.set_metadata(aggregate,
+                                                   unset_property)
