@@ -13,40 +13,47 @@
 
 """Identity v3 IdentityProvider action implementations"""
 
+import logging
+
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import utils
+from openstackclient.i18n import _
+
+
+LOG = logging.getLogger(__name__)
 
 
 class CreateIdentityProvider(command.ShowOne):
-    """Create new identity provider"""
+    _description = _("Create new identity provider")
 
     def get_parser(self, prog_name):
         parser = super(CreateIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
             'identity_provider_id',
             metavar='<name>',
-            help='New identity provider name (must be unique)'
+            help=_('New identity provider name (must be unique)'),
         )
         identity_remote_id_provider = parser.add_mutually_exclusive_group()
         identity_remote_id_provider.add_argument(
             '--remote-id',
             metavar='<remote-id>',
             action='append',
-            help='Remote IDs to associate with the Identity Provider '
-                 '(repeat option to provide multiple values)'
+            help=_('Remote IDs to associate with the Identity Provider '
+                   '(repeat option to provide multiple values)'),
         )
         identity_remote_id_provider.add_argument(
             '--remote-id-file',
             metavar='<file-name>',
-            help='Name of a file that contains many remote IDs to associate '
-                 'with the identity provider, one per line'
+            help=_('Name of a file that contains many remote IDs to associate '
+                   'with the identity provider, one per line'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='New identity provider description',
+            help=_('New identity provider description'),
         )
         enable_identity_provider = parser.add_mutually_exclusive_group()
         enable_identity_provider.add_argument(
@@ -54,13 +61,13 @@ class CreateIdentityProvider(command.ShowOne):
             dest='enabled',
             action='store_true',
             default=True,
-            help='Enable identity provider (default)',
+            help=_('Enable identity provider (default)'),
         )
         enable_identity_provider.add_argument(
             '--disable',
             dest='enabled',
             action='store_false',
-            help='Disable the identity provider',
+            help=_('Disable the identity provider'),
         )
         return parser
 
@@ -87,25 +94,39 @@ class CreateIdentityProvider(command.ShowOne):
 
 
 class DeleteIdentityProvider(command.Command):
-    """Delete identity provider"""
+    _description = _("Delete identity provider(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
             'identity_provider',
             metavar='<identity-provider>',
-            help='Identity provider to delete',
+            nargs='+',
+            help=_('Identity provider(s) to delete'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        identity_client.federation.identity_providers.delete(
-            parsed_args.identity_provider)
+        result = 0
+        for i in parsed_args.identity_provider:
+            try:
+                identity_client.federation.identity_providers.delete(i)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete identity providers with "
+                          "name or ID '%(provider)s': %(e)s")
+                          % {'provider': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.identity_provider)
+            msg = (_("%(result)s of %(total)s identity providers failed"
+                   " to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListIdentityProvider(command.Lister):
-    """List identity providers"""
+    _description = _("List identity providers")
 
     def take_action(self, parsed_args):
         columns = ('ID', 'Enabled', 'Description')
@@ -119,58 +140,50 @@ class ListIdentityProvider(command.Lister):
 
 
 class SetIdentityProvider(command.Command):
-    """Set identity provider properties"""
+    _description = _("Set identity provider properties")
 
     def get_parser(self, prog_name):
         parser = super(SetIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
             'identity_provider',
             metavar='<identity-provider>',
-            help='Identity provider to modify',
+            help=_('Identity provider to modify'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='Set identity provider description',
+            help=_('Set identity provider description'),
         )
         identity_remote_id_provider = parser.add_mutually_exclusive_group()
         identity_remote_id_provider.add_argument(
             '--remote-id',
             metavar='<remote-id>',
             action='append',
-            help='Remote IDs to associate with the Identity Provider '
-                 '(repeat option to provide multiple values)'
+            help=_('Remote IDs to associate with the Identity Provider '
+                   '(repeat option to provide multiple values)'),
         )
         identity_remote_id_provider.add_argument(
             '--remote-id-file',
             metavar='<file-name>',
-            help='Name of a file that contains many remote IDs to associate '
-                 'with the identity provider, one per line'
+            help=_('Name of a file that contains many remote IDs to associate '
+                   'with the identity provider, one per line'),
         )
         enable_identity_provider = parser.add_mutually_exclusive_group()
         enable_identity_provider.add_argument(
             '--enable',
             action='store_true',
-            help='Enable the identity provider',
+            help=_('Enable the identity provider'),
         )
         enable_identity_provider.add_argument(
             '--disable',
             action='store_true',
-            help='Disable the identity provider',
+            help=_('Disable the identity provider'),
         )
         return parser
 
     def take_action(self, parsed_args):
         federation_client = self.app.client_manager.identity.federation
 
-        # Basic argument checking
-        if (not parsed_args.enable and not parsed_args.disable and
-                not parsed_args.remote_id and
-                not parsed_args.remote_id_file and
-                not parsed_args.description):
-            self.log.error('No changes requested')
-            return (None, None)
-
         # Always set remote_ids if either is passed in
         if parsed_args.remote_id_file:
             file_content = utils.read_blob_file_contents(
@@ -191,22 +204,21 @@ class SetIdentityProvider(command.Command):
         if parsed_args.remote_id_file or parsed_args.remote_id:
             kwargs['remote_ids'] = remote_ids
 
-        identity_provider = federation_client.identity_providers.update(
-            parsed_args.identity_provider, **kwargs)
-
-        identity_provider._info.pop('links', None)
-        return zip(*sorted(six.iteritems(identity_provider._info)))
+        federation_client.identity_providers.update(
+            parsed_args.identity_provider,
+            **kwargs
+        )
 
 
 class ShowIdentityProvider(command.ShowOne):
-    """Display identity provider details"""
+    _description = _("Display identity provider details")
 
     def get_parser(self, prog_name):
         parser = super(ShowIdentityProvider, self).get_parser(prog_name)
         parser.add_argument(
             'identity_provider',
             metavar='<identity-provider>',
-            help='Identity provider to display',
+            help=_('Identity provider to display'),
         )
         return parser
 
@@ -214,7 +226,8 @@ class ShowIdentityProvider(command.ShowOne):
         identity_client = self.app.client_manager.identity
         idp = utils.find_resource(
             identity_client.federation.identity_providers,
-            parsed_args.identity_provider)
+            parsed_args.identity_provider,
+            id=parsed_args.identity_provider)
 
         idp._info.pop('links', None)
         remote_ids = utils.format_list(idp._info.pop('remote_ids', []))
